#ifndef __VERTEXDATA_H__
#define __VERTEXDATA_H__

namespace Geometry
{
	class VertexData
	{
	private:
		unsigned int numverts;
	public:
		VertexData(unsigned int numv):numverts(numv){}
		virtual ~VertexData(void){}

		virtual void Begin(void)=0;
		virtual void SetVertexCoord(unsigned int i, float x, float y, float z)=0;
		virtual void SetVertexNormal(unsigned int i, float x, float y, float z)=0;
		virtual void SetVertexTexCoord(unsigned int i, float x, float y)=0;
		virtual void End(void)=0;	

		virtual void GetVertexCoord(unsigned int i, float &x, float &y, float &z) const = 0;
		virtual void GetVertexNormal(unsigned int i, float &x, float &y, float &z) const = 0;
		virtual void GetVertexTexCoord(unsigned int i, float &x, float &y) const =0;

		unsigned int GetNumVertices(void) const { return numverts; }
	};

	class IndexData
	{
	private:
		unsigned int numinds, numvalidinds;
	public:
		IndexData(unsigned int numi):numinds(numi),numvalidinds(numi){}
		virtual ~IndexData(void){}

		virtual void Begin(void)=0;
		virtual void SetIndex(unsigned int i, unsigned int index)=0;
		virtual void End(void)=0;

		virtual unsigned int GetIndex(unsigned int) const =0;
		unsigned int GetNumMaxIndices(void) const { return numinds; }
		unsigned int GetNumValidIndices(void) const { return numvalidinds; }
		void SetNumValidIndices(unsigned int valid){ numvalidinds=valid; }
	};

	class MultiIndexData
	{
	private:
		unsigned int num_prims, num_total_inds, *num_max_inds, *num_valid_inds;
	public:
		MultiIndexData(unsigned int numprims, unsigned int *maxinds){
			num_prims=numprims;
			num_total_inds=0;
			num_max_inds=new unsigned int[numprims];
			num_valid_inds=new unsigned int[numprims];
			for (unsigned int i=0; i<numprims; i++)
			{
				num_max_inds[i]=num_valid_inds[i]=maxinds[i];
				num_total_inds += maxinds[i];
			}
		}

		~MultiIndexData(void){
			delete[] num_max_inds;
			delete[] num_valid_inds;
		}

		virtual void Begin(void)=0;
		virtual void SetIndex(unsigned int iprim, unsigned int i, unsigned int index)=0;
		virtual void End(void)=0;

		virtual unsigned int GetIndex(unsigned int iprim, unsigned int index) const =0;
		unsigned int GetNumMaxIndices(unsigned int iprim) const { return num_max_inds[iprim]; }
		unsigned int GetNumValidIndices(unsigned int iprim) const { return num_valid_inds[iprim]; }
		void SetNumValidIndices(unsigned int iprim, unsigned int valid){ num_valid_inds[iprim]=valid; }
		unsigned int GetNumPrims(void) const { return num_prims; }
	};

	typedef VertexData* CreateVertexDataFunc(unsigned int numv);
	typedef CreateVertexDataFunc* CREATEVERTEXDATAFUNC;
	typedef IndexData* CreateIndexDataFunc(unsigned int numi);
	typedef CreateIndexDataFunc* CREATEINDEXDATAFUNC;
	typedef MultiIndexData* CreateMultiIndexDataFunc(unsigned int numprims, unsigned int *numi);
	typedef CreateMultiIndexDataFunc* CREATEMULTIINDEXDATAFUNC;

	class DefaultVertexData : public VertexData
	{
	private:
		float *v, *n, *t;
	public:
		DefaultVertexData(unsigned int numv);
		~DefaultVertexData(void);

		virtual void Begin(void){}
		virtual void SetVertexCoord(unsigned int i, float x, float y, float z){ 
			v[i*3+0]=x; v[i*3+1]=y; v[i*3+2]=z; }
		virtual void SetVertexNormal(unsigned int i, float x, float y, float z){
			n[i*3+0]=x; n[i*3+1]=y; n[i*3+2]=z; }
		virtual void SetVertexTexCoord(unsigned int i, float x, float y){ 
			t[i*2+0]=x; t[i*2+1]=y; }

		virtual void End(void){}

		virtual void GetVertexCoord(unsigned int i, float &x, float &y, float &z) const { 
			x=v[i*3+0]; 
			y=v[i*3+1]; 
			z=v[i*3+2];
		}
		virtual void GetVertexNormal(unsigned int i, float &x, float &y, float &z) const {x=n[i*3+0]; y=n[i*3+1]; z=n[i*3+2]; }
		virtual void GetVertexTexCoord(unsigned int i, float &x, float &y) const {x=t[i*2+0]; y=t[i*2+1]; }

		const float * GetVertexFloatBuffer(void) const { return v; }

	};
	VertexData *DefaultVertexDataCreator(unsigned int numv);


	class DefaultIndexData : public IndexData
	{
	private:
		unsigned int *indices;
	public:
		DefaultIndexData(unsigned int numi);
		~DefaultIndexData(void);

		virtual void Begin(void){}
		virtual void SetIndex(unsigned int i, unsigned int index){ 
			indices[i] = index; 
		}
		virtual void End(void){}

		virtual unsigned int GetIndex(unsigned int i) const {
			return indices[i];
		}

	};
	IndexData *DefaultIndexDataCreator(unsigned int numi);

	class DefaultMultiIndexData : public MultiIndexData
	{
	private:
		unsigned int **indices;
	public:
		DefaultMultiIndexData(unsigned int numprims, unsigned int *numi);
		~DefaultMultiIndexData(void);

		virtual void Begin(void){}
		virtual void SetIndex(unsigned int iprim, unsigned int i, unsigned int index){ 
			indices[iprim][i] = index; 
		}
		virtual void End(void){}

		virtual unsigned int GetIndex(unsigned int iprim, unsigned int i) const {
			return indices[iprim][i];
		}

	};
	MultiIndexData *DefaultMultiIndexDataCreator(unsigned int numprims, unsigned int *numi);

}

#endif